/*
Copyright 2024 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package awsup

// KopsResourceRevisionTag is the tag used to store the revision timestamp,
// when we are forced to create a new version of a resource because we cannot modify it in-place.
// This happens when the resource field is immutable;
// it also happens for ELBs, when we cannot have two ELBs pointing at the same target group
// and thus must create a second.
const KopsResourceRevisionTag = "kops.k8s.io/revision"
