/*
Copyright 2019 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package resources

import (
	"k8s.io/kops/upup/pkg/fi"
)

type Resource struct {
	Name string
	Type string
	ID   string

	// If true, this resource is not owned by the cluster
	Shared bool

	Blocks  []string
	Blocked []string
	Done    bool

	Deleter      func(cloud fi.Cloud, tracker *Resource) error
	GroupKey     string
	GroupDeleter func(cloud fi.Cloud, trackers []*Resource) error

	// Dumper populates the dump with any information from the resource
	Dumper func(op *DumpOperation, r *Resource) error

	Obj interface{}
}
