// Code generated by smithy-go-codegen DO NOT EDIT.

package cloudwatchlogs

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/cloudwatchlogs/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Returns a list of data source associations for a specified S3 Table
// Integration, showing which data sources are currently associated for query
// access.
func (c *Client) ListSourcesForS3TableIntegration(ctx context.Context, params *ListSourcesForS3TableIntegrationInput, optFns ...func(*Options)) (*ListSourcesForS3TableIntegrationOutput, error) {
	if params == nil {
		params = &ListSourcesForS3TableIntegrationInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListSourcesForS3TableIntegration", params, optFns, c.addOperationListSourcesForS3TableIntegrationMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListSourcesForS3TableIntegrationOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListSourcesForS3TableIntegrationInput struct {

	// The Amazon Resource Name (ARN) of the S3 Table Integration to list associations
	// for.
	//
	// This member is required.
	IntegrationArn *string

	// The maximum number of associations to return in a single call. Valid range is 1
	// to 100.
	MaxResults *int32

	// The token for the next set of items to return. The token expires after 24 hours.
	NextToken *string

	noSmithyDocumentSerde
}

type ListSourcesForS3TableIntegrationOutput struct {

	// The token for the next set of items to return. The token expires after 24 hours.
	NextToken *string

	// The list of data source associations for the specified S3 Table Integration.
	Sources []types.S3TableIntegrationSource

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListSourcesForS3TableIntegrationMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpListSourcesForS3TableIntegration{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpListSourcesForS3TableIntegration{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListSourcesForS3TableIntegration"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addSpanRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addTimeOffsetBuild(stack, c); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = addCredentialSource(stack, options); err != nil {
		return err
	}
	if err = addOpListSourcesForS3TableIntegrationValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListSourcesForS3TableIntegration(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addInterceptBeforeRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addInterceptAttempt(stack, options); err != nil {
		return err
	}
	if err = addInterceptors(stack, options); err != nil {
		return err
	}
	return nil
}

// ListSourcesForS3TableIntegrationPaginatorOptions is the paginator options for
// ListSourcesForS3TableIntegration
type ListSourcesForS3TableIntegrationPaginatorOptions struct {
	// The maximum number of associations to return in a single call. Valid range is 1
	// to 100.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListSourcesForS3TableIntegrationPaginator is a paginator for
// ListSourcesForS3TableIntegration
type ListSourcesForS3TableIntegrationPaginator struct {
	options   ListSourcesForS3TableIntegrationPaginatorOptions
	client    ListSourcesForS3TableIntegrationAPIClient
	params    *ListSourcesForS3TableIntegrationInput
	nextToken *string
	firstPage bool
}

// NewListSourcesForS3TableIntegrationPaginator returns a new
// ListSourcesForS3TableIntegrationPaginator
func NewListSourcesForS3TableIntegrationPaginator(client ListSourcesForS3TableIntegrationAPIClient, params *ListSourcesForS3TableIntegrationInput, optFns ...func(*ListSourcesForS3TableIntegrationPaginatorOptions)) *ListSourcesForS3TableIntegrationPaginator {
	if params == nil {
		params = &ListSourcesForS3TableIntegrationInput{}
	}

	options := ListSourcesForS3TableIntegrationPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListSourcesForS3TableIntegrationPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListSourcesForS3TableIntegrationPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListSourcesForS3TableIntegration page.
func (p *ListSourcesForS3TableIntegrationPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListSourcesForS3TableIntegrationOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	optFns = append([]func(*Options){
		addIsPaginatorUserAgent,
	}, optFns...)
	result, err := p.client.ListSourcesForS3TableIntegration(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

// ListSourcesForS3TableIntegrationAPIClient is a client that implements the
// ListSourcesForS3TableIntegration operation.
type ListSourcesForS3TableIntegrationAPIClient interface {
	ListSourcesForS3TableIntegration(context.Context, *ListSourcesForS3TableIntegrationInput, ...func(*Options)) (*ListSourcesForS3TableIntegrationOutput, error)
}

var _ ListSourcesForS3TableIntegrationAPIClient = (*Client)(nil)

func newServiceMetadataMiddleware_opListSourcesForS3TableIntegration(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListSourcesForS3TableIntegration",
	}
}
